/*
 * cpufreqd-icon (C) 2009-2017 kakurasan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

using Config;
using GLib.Intl;
using Gtk;
using CpufreqdIcon.Function;

namespace CpufreqdIcon
{
  class App : Gtk.Application
  {
    public static bool running = false;
    public static App inst;
    public static TrayIcon status_icon;
    public static CpufreqdRemote remote;
    public static Notification notification;
    public static Gdk.Pixbuf? icon;
    public const string comments = _("System tray icon for cpufreqd");
    public const string copyright = "(C) 2009-2017 kakurasan";
    public const string[] authors =
    {
      "cpufreqd-icon - Copyright (C) 2009-2017",
      " kakurasan <kakurasan@gmail.com>",
      "",
      "cpufreqd - Copyright (C) 2002-2009",
      " Mattia Dongili <malattia@linux.it>",
      " George Staikos <staikos@0wned.org>",
      null
    };
    public const string license = _("This program is licensed under GPL-2 or lator");
    public
    App ()
    {
      GLib.Environment.set_application_name ("cpufreqd-icon");
      GLib.Object (application_id : "com.blogspot.kakurasan.cpufreqd-icon");
    }
    protected override void
    startup ()
    {
      bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
      bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
      textdomain (GETTEXT_PACKAGE);

      try
      {
        App.icon = new Gdk.Pixbuf.from_file (GLib.Path.build_filename (PACKAGE_DATA_DIR, "pixmaps", "%s.png".printf (PACKAGE)));
      }
      catch (GLib.Error e)
      {
        App.icon = null;
      }

      base.startup ();
    }
    protected override void
    activate ()
    {
      if (App.running)
        return;
      App.running = true;

      App.remote = new CpufreqdRemote ();
#if DEBUG
      try
      {
        App.remote.find_socket ();
      }
      catch (GLib.FileError e)
      {
        GLib.debug ("%s", e.message);
        Function.run_messagedialog (Gtk.MessageType.WARNING,
                                    App.icon,
                                    null,
                                    "No cpufreqd sockets found",
                                    "Cannot find cpufreqd socket");
      }
#endif
      App.status_icon = new TrayIcon ();
      App.notification = new Notification ();
      base.activate ();
      Gtk.main ();
    }
    protected override void
    shutdown ()
    {
      base.shutdown ();
    }
  }
  public static int
  main (string[] args)
  {
    App.inst = new App ();
    return App.inst.run (args);
  }
}
