/*
 * cpufreqd-icon (C) 2009-2017 kakurasan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

namespace CpufreqdIcon
{
  class CpufreqdRemote
  {
    string sockpath;
    GLib.Socket sock;
    /* cpufreqd remote interface */
    public const uint32 CMD_SHIFT         = 16;
//    public const uint32 CMD_UPDATE_STATE  = 1;
    public const uint32 CMD_SET_PROFILE   = 2;  /* profile index */
    public const uint32 CMD_LIST_PROFILES = 3;
//    public const uint32 CMD_SET_RULE      = 4;  /* rule index */
//    public const uint32 CMD_LIST_RULES    = 5;
    public const uint32 CMD_SET_MODE      = 6;  /* mode */
//    public const uint32 CMD_CUR_PROFILES  = 7;
//    public const uint32 ARG_MASK          = (uint32) 0x0000ffff;
    public const uint32 MODE_DYNAMIC      = 1;
    public const uint32 MODE_MANUAL       = 2;
//    public const uint32 INVALID_CMD       = (uint32) 0xffffffff;
//    public static uint32 remote_cmd (uint32 c)
//    {
//      return (c >> CpufreqdRemote.CMD_SHIFT);
//    }
//    public static uint32 remote_arg (uint32 c)
//    {
//      return (c & CpufreqdRemote.ARG_MASK);
//    }
    public static uint32
    make_command (uint32 cmd, uint32 arg)
    {
      return ((cmd << CpufreqdRemote.CMD_SHIFT) | arg);
    }
    public void
    find_socket () throws GLib.FileError
    {
      GLib.Dir dir;
      string tmpsockpath;
      weak string item = null;
      time_t mtime_last = 0;
      Posix.Stat statbuf;
      dir = GLib.Dir.open ("/tmp", 0);
      this.sockpath = null;
      for (;;)
      {
        item = dir.read_name ();
        if (item == null)
          break;
        if (item.has_prefix ("cpufreqd-"))
        {
#if DEBUG
          GLib.debug ("\"%s\".has_prefix(\"cpufreqd-\") == true", item);
#endif
          tmpsockpath = GLib.Path.build_path ("/", "/tmp", item, "cpufreqd");
          if (Posix.stat (tmpsockpath, out statbuf) == 0)
          {
            if (statbuf.st_mtime > mtime_last)
            {
#if DEBUG
              GLib.debug ("newest socket(%ld > %ld): %s",
                          statbuf.st_mtime, mtime_last, tmpsockpath);
#endif
              mtime_last = statbuf.st_mtime;
              this.sockpath = tmpsockpath;
            }
          }
#if DEBUG
          else
            GLib.debug ("cannot stat \"%s\"", tmpsockpath);
#endif
        }  // if (item.has_prefix ("cpufreqd-"))
#if DEBUG
        GLib.debug ("%s", item);
#endif
      }  // for (;;)
      if (this.sockpath == null)
        throw new GLib.FileError.FAILED ("no cpufreqd sockets found");
#if DEBUG
      GLib.debug ("using socket \"%s\"", this.sockpath);
#endif
    }
    public bool
    connect ()
    {
      try
      {
        this.sock = new GLib.Socket (GLib.SocketFamily.UNIX,
                                     GLib.SocketType.STREAM,
                                     GLib.SocketProtocol.DEFAULT);
        if (! this.sock.connect (new GLib.UnixSocketAddress (this.sockpath), null))
        {
          try
          {
            this.find_socket ();
          }
          catch (GLib.FileError e)
          {
            return false;
          }
          this.sock = new GLib.Socket (GLib.SocketFamily.UNIX,
                                       GLib.SocketType.STREAM,
                                       GLib.SocketProtocol.DEFAULT);
          if (! this.sock.connect (new GLib.UnixSocketAddress (this.sockpath), null))
            return false;
        }
      }
      catch (GLib.Error e)
      {
#if DEBUG
        GLib.debug ("%s", e.message);
#endif
        return false;
      }
#if DEBUG
      GLib.debug ("connected");
#endif
      return true;
    }
    public void
    disconnect ()
    {
      this.sock = null;
#if DEBUG
      GLib.debug ("disconnected");
#endif
    }
    public bool
    query_profiles ()
    {
      GLib.IOChannel ioch;
      uint32 cmd;
      string line;
      string pfname = string.nfill (255, '\0');  // 255 = MAX_STRING_LEN in src/cpufreqd.h
      string govname = string.nfill (255, '\0');
      size_t length, terminator_pos;
      uint active, minfreq, maxfreq;
      int i = 1;
      CpufreqdProfile profile;
      CpufreqdProfile.profiles = new SList<CpufreqdProfile> ();
      cmd = CpufreqdRemote.make_command (CpufreqdRemote.CMD_LIST_PROFILES, 0);
      if (! this.connect ())
      {
#if DEBUG
        GLib.debug ("connect failed");
#endif
        return false;
      }
      if (Posix.write (this.sock.fd, &cmd, 4) != 4)
      {
#if DEBUG
        GLib.debug ("Posix.write() failed");
#endif
        this.disconnect ();
        return false;
      }
      ioch = new GLib.IOChannel.unix_new (this.sock.fd);
      try
      {
        ioch.set_encoding (null);
        ioch.set_flags (GLib.IOFlags.IS_READABLE);
      }
      catch (GLib.IOChannelError e)
      {
        ;
      }
      for (;;)
      {
        try
        {
          if (ioch.read_line (out line, out length, out terminator_pos) != GLib.IOStatus.NORMAL)
            break;
        }
        catch (GLib.ConvertError e)
        {
          GLib.warning ("ConvertError: %s", e.message);
        }
        catch (GLib.IOChannelError e)
        {
          try
          {
            ioch.shutdown (false);
          }
          catch (GLib.IOChannelError e)
          {
            ;
          }
          finally
          {
            this.disconnect ();
          }
          return false;
        }
#if DEBUG
        GLib.debug ("line: [%s]", line);
#endif
        /* active(0 or 1), profile name, min freq, max freq, governor name */
        if (line.scanf ("%d/%[^/]/%u/%u/%[^\n]\n", out active, pfname, out minfreq, out maxfreq, govname) != 5)
        {
#if DEBUG
          GLib.debug ("scanf() != 5");
#endif
          break;
        }
        profile = new CpufreqdProfile (i, pfname, govname, minfreq / 1000, maxfreq / 1000);
        /* prepend to SList */
        CpufreqdProfile.profiles.prepend (profile);
        i++;
      }  // for (;;)
      try
      {
#if DEBUG
        GLib.debug ("ioch.shutdown ()");
#endif
        ioch.shutdown (false);
      }
      catch (GLib.IOChannelError e)
      {
        ;
      }
      finally
      {
        this.disconnect ();
      }
      return true;
    }
    bool
    send_commands (uint32 cmd1, uint32? cmd2 = null)
    {
      uint32 cmd = cmd1;
      if (this.connect () == false)
      {
#if DEBUG
        GLib.debug ("connect failed");
#endif
        return false;
      }
      ssize_t bytes_written = Posix.write (this.sock.fd, &cmd, 4);
      this.disconnect ();
      if (bytes_written != 4)
        return false;
      if (cmd2 != null)
      {
        cmd = cmd2;
        if (this.connect () == false)
        {
#if DEBUG
        GLib.debug ("connect failed");
#endif
          return false;
        }
        bytes_written = Posix.write (this.sock.fd, &cmd, 4);
        this.disconnect ();
        if (bytes_written != 4)
          return false;
      }
      return true;
    }
    public bool
    set_dynamic ()
    {
      string msg;
      if (this.send_commands (CpufreqdRemote.make_command (CpufreqdRemote.CMD_SET_MODE, CpufreqdRemote.MODE_DYNAMIC)))
      {
        msg = _("Switched to dynamic mode");
        App.notification.show (GLib.NotificationPriority.NORMAL, _("Dynamic mode"), msg);
        return true;
      }
      else
      {
        msg = _("Could not switch to dynamic mode");
#if DEBUG
        GLib.debug ("%s", msg);
#endif
        App.notification.show (GLib.NotificationPriority.URGENT, _("Error"), msg);
        return false;
      }
    }
    public void
    set_profile (ProfileMenuItem item)
    {
      string msg;
      CpufreqdProfile p = item.profile;
      if (this.send_commands (CpufreqdRemote.make_command (CpufreqdRemote.CMD_SET_MODE, CpufreqdRemote.MODE_MANUAL), CpufreqdRemote.make_command (CpufreqdRemote.CMD_SET_PROFILE, p.num)))
      {
        msg = _("Profile has been changed.\n Name: %s\n Governor: %s\n Clock range: %u-%u MHz").printf (p.pfname, p.govname, p.minfreq, p.maxfreq);
        App.notification.show (GLib.NotificationPriority.NORMAL, _("Manual profile change"), msg);
      }
      else
      {
        msg = _("Could not set profile");
        App.notification.show (GLib.NotificationPriority.URGENT, _("Error"), msg);
      }
    }
  }
}
