/*
 * cpufreqd-icon (C) 2009-2017 kakurasan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

using AppIndicator;
using Config;
using Gtk;

namespace CpufreqdIcon
{
  class TrayIcon
  {
    AppIndicator.Indicator indicator;
    public
    TrayIcon ()
    {
      this.indicator = new AppIndicator.Indicator (PACKAGE, PACKAGE, AppIndicator.IndicatorCategory.SYSTEM_SERVICES);
      this.indicator.set_status (AppIndicator.IndicatorStatus.ACTIVE);
      this.build_menu ();
    }
    void
    build_menu ()
    {
      var menu = new Gtk.Menu ();
      var item_quit = new Gtk.MenuItem.with_mnemonic (_("_Quit"));
      item_quit.activate.connect (Gtk.main_quit);
      menu.prepend (item_quit);
      menu.prepend (new Gtk.SeparatorMenuItem ());
      var item_about = new Gtk.MenuItem.with_mnemonic (_("Abou_t cpufreqd-icon"));
      item_about.activate.connect (() =>
      {
        Gtk.show_about_dialog (null,
                               "title", _("About cpufreqd-icon"),
                               "logo-icon-name", PACKAGE,
                               "program-name", PACKAGE,
                               "version", VERSION,
                               "comments", App.comments,
                               "copyright", App.copyright,
                               "authors", App.authors,
                               "license", App.license,
                               "wrap-license", true,
                               "website", PACKAGE_URL,
                               null);
      });
      menu.prepend (item_about);

      var item_reloadinfo = new Gtk.MenuItem.with_mnemonic (_("_Reload info"));
      item_reloadinfo.activate.connect (() =>
      {
        this.build_menu ();
      });
      menu.prepend (item_reloadinfo);

      try
      {
        App.remote.find_socket ();
        if (App.remote.query_profiles ())
        {
          menu.prepend (new Gtk.SeparatorMenuItem ());
          var item_dynamic = new Gtk.MenuItem.with_mnemonic (_("Dymamic _mode"));
          menu.prepend (item_dynamic);
          menu.prepend (new Gtk.SeparatorMenuItem ());
          foreach (CpufreqdProfile p in CpufreqdProfile.profiles)
          {
            var item = new ProfileMenuItem (p);
            item.activate.connect ((source) => App.remote.set_profile ((ProfileMenuItem) source));
            menu.prepend (item);
          }
          item_dynamic.activate.connect (() => App.remote.set_dynamic ());
        }
        else
          throw new GLib.FileError.FAILED ("query failed");
      }
      catch (GLib.FileError e)
      {
        ;
      }

      menu.show_all ();
      this.indicator.set_menu (menu);
    }
  }
}
